import lxdatacleaner as dc
import pandas as pd
'''
input = dc.input()

path = 'D:\\project\\dataclean\\test\\b.csv'
a = input.read_csv('D:\\project\\dataclean\\test\\b.csv', 'utf-8')
# print(a.head())
ll = ['轮式拖拉机']
domain = dc.domain()
b = domain.in_domain(df=a, column='jizhong', list=ll)
print(b.head())
output = dc.output()
#output.to_csv(b, path, 'utf-8')
check = dc.check()

c = check.check_Phonenum(b,'jixing')
print(c[c['PhoneNumber_check']==True].head())
check.check_address.check_sheng(a,'zhong')
'''
a="{'id': '11', 'sheng': '安徽省', 'shi': '六安市', 'xian': '霍邱县', 'zhen': '高塘镇', 'cun': '八里村赵店组', 'name': '曾凡明', 'jijupinmu': '自走履带式谷物联合收割机（全喂入）', 'shengchanchangjia': '星光农机股份有限公司', 'chanpinmingcheng': '全喂入联合收割机', 'goumaijixing': '4LZ-5.0Z', 'goumaishuliang': '1', 'jingxiaoshang': '霍邱县承军商贸有限公司(经销商)', 'date': '43733', 'dantaishoujia': '100000.0', 'dantaibutie': '28000', 'butiezonge': '28000.0', 'zhuangtai': '已提交财政部门未结算', 'jizhong': '全喂入收割机', 'jixingfenlei': '5≤喂入量＜6kg/s'}"
import json
#print(a.replace(",",'"'))
b = json.loads(a.replace("'",'"'))
print(b)
keys = []

for dic in b:
    keys.append(dic)
    #print(dic)
    #print(b[dic])
    print(keys)
print(keys)
    #keys = dic.keys()
    #print(keys)


# -*- coding: utf-8 -*-
import pandas as pd

'''
@author:Hao Qi

@version:0.0.1
'''
"""
Copyright © 2020 <copyright holders>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the “Software”), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""


# 数据定义域类，输入数据，输出是否在该定义域中
class domain(object):

    def between(self,df,column,low: float,top: float):
        try:
            df = df[df[column]>=low & df[column]<=top]
            return df
        except:
            print('This domain option error')
            df=pd.DataFrame()
            return df
        

    def not_between(self,df,column,low: float,top: float):
        try:
            df = df[df[column]<=low&df[column]>=top]
            return df
        except:
            print('This domain option error')
            df=pd.DataFrame()
            return df
        

    def in_domain(self,df,column,list):
        try:
            df = df[df[column].isin(list)]
            return df
        except:
            print('This domain option error')
            df=pd.DataFrame()
            return df


    def out_domain(self,df,column,list):
        try:
            df = df[df[column].notin(list)]
            return df
        except:
            print('This domain option error')
            df=pd.DataFrame()
            return df



# -*- coding: utf-8 -*-
import pandas as pd
import re
'''
@author:Hao Qi

@version:0.0.1
'''
"""
Copyright © 2020 <copyright holders>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the “Software”), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

class check(object):

    def check_address(self):
        a = self.check_address()
    def check_Phonenum(self,df,column):
        try:
            phone_pattern = re.compile(r'^(13\d|14[5|7]|15\d|166|17[3|6|7]|18\d)\d{8}$')
            tmp = pd.DataFrame()
            tmp[column]=df[column]
            tmp['PhoneNumber_check'] = df[column].apply(lambda x: True if phone_pattern.match(x) else False)
            df = pd.merge(df,tmp)
            return df
        except:
            print('check phone number error')
            df = pd.DataFrame()
            return df
    def check_Email(self,df,column):
        try:
            phone_pattern = re.compile(r'^[0-9a-zA-Z_]{0,19}@[0-9a-zA-Z]{1,13}\.[com,cn,net]{1,3}$')
            tmp = pd.DataFrame()
            tmp[column]=df[column]
            tmp['Email_check'] = df[column].apply(lambda x: True if phone_pattern.match(x) else False)
            df = pd.merge(df,tmp)
            return df
        except:
            print('check email error')
            df = pd.DataFrame()
            return df

    def check_name(self,df,column):
        try:
            error_key=['.','·']
            tmp=pd.DataFrame()
            tmp[column]=df[column]
            tmp['is_name']=df[column].apply(lambda x: True if len(x)<5 and x not in error_key else False)
            df = pd.merge(df, tmp)
            return df
        except:
            print('check name error')
            df = pd.DataFrame()
            return df

class check_address(check):

    def check_sheng(self,df,column):
        try:
            sheng=['安徽省','北京市','重庆市','福建省','广东省','广东省农垦总局','甘肃省','广西壮族自治区','贵州省','湖北省',
                       '河北省','黑龙江省','黑龙江农垦','湖南省','海南省','河南省','吉林省','江苏省','江西省','辽宁省',
                       '内蒙古自治区','宁夏回族自治区','青海省','四川省','山东省','上海市','山西省','陕西省','天津市','西藏自治区',
                       '新疆生产建设兵团','新疆维吾尔自治区','云南省','浙江省','香港特别行政区','澳门特别行政区','台湾省']
            tmp = pd.DataFrame()
            tmp[column] = df[column]
            tmp['sheng_check'] = df[column].apply(lambda x: True if x in sheng else False)
            df = pd.merge(df, tmp)
            return df
        except:
            print('check address error')
            df = pd.DataFrame()
            return df

    def check_shi(self,df,column):
        try:
            shi = ['北京市','天津市','石家庄市','唐山市','秦皇岛市','邯郸市','邢台市','保定市','张家口市','承德市',
                    '沧州市','廊坊市','衡水市','太原市','大同市','阳泉市','长治市','晋城市','朔州市','晋中市','运城市',
                    '忻州市','临汾市','吕梁市','呼和浩特市','包头市','乌海市','赤峰市','通辽市','鄂尔多斯市','呼伦贝尔市','巴彦淖尔市',
                    '乌兰察布市','兴安盟','锡林郭勒盟','阿拉善盟','沈阳市','大连市','鞍山市','抚顺市','本溪市','丹东市','锦州市','营口市',
                    '阜新市','辽阳市','盘锦市','铁岭市','朝阳市','葫芦岛市','长春市','吉林市','四平市','辽源市','通化市','白山市','松原市',
                    '白城市','延边朝鲜族自治州','哈尔滨市','齐齐哈尔市','鸡西市','鹤岗市','双鸭山市','大庆市','伊春市','佳木斯市','七台河市',
                    '牡丹江市','黑河市','绥化市','大兴安岭地区','上海市','南京市','无锡市','徐州市','常州市','苏州市','南通市','连云港市',
                    '淮安市','盐城市','扬州市','镇江市','泰州市','宿迁市','杭州市','宁波市','温州市','嘉兴市','湖州市','绍兴市','金华市',
                    '衢州市','舟山市','台州市','丽水市','合肥市','芜湖市','蚌埠市','淮南市','马鞍山市','淮北市','铜陵市','安庆市','黄山市',
                    '滁州市','阜阳市','宿州市','六安市','亳州市','池州市','宣城市','福州市','厦门市','莆田市','三明市','泉州市','漳州市','南平市',
                    '龙岩市','宁德市','南昌市','景德镇市','萍乡市','九江市','新余市','鹰潭市','赣州市','吉安市','宜春市','抚州市','上饶市','济南市',
                    '青岛市','淄博市','枣庄市','东营市','烟台市','潍坊市','济宁市','泰安市','威海市','日照市','莱芜市','临沂市','德州市','聊城市',
                    '滨州市','菏泽市','郑州市','开封市','洛阳市','平顶山市','安阳市','鹤壁市','新乡市','焦作市''济源市','濮阳市','许昌市','漯河市',
                    '三门峡市','南阳市','商丘市','信阳市','周口市','驻马店市','武汉市','黄石市''十堰市','宜昌市','襄阳市','鄂州市','荆门市','孝感市',
                    '荆州市','黄冈市','咸宁市','随州市','恩施土家族苗族自治州','仙桃市','潜江市','天门市','神农架林区','长沙市','株洲市','湘潭市','衡阳市',
                    '邵阳市','岳阳市','常德市','张家界市','益阳市','郴州市','永州市','怀化市','娄底市','湘西土家族苗族自治州','广州市','韶关市','深圳市','珠海市',
                    '汕头市',  '佛山市','江门市','湛江市','茂名市','肇庆市','惠州市','梅州市','汕尾市','河源市','阳江市','清远市','东莞市','中山市','东沙群岛',
                    '潮州市','揭阳市','云浮市','南宁市','柳州市','桂林市','梧州市','北海市','防城港市','钦州市','贵港市','玉林市','百色市','贺州市','河池市','来宾市',
                    '崇左市','海口市','三亚市','三沙市','五指山市','琼海市','儋州市','文昌市','万宁市','东方市','定安县','屯昌县','澄迈县','临高县','白沙黎族自治县',
                    '昌江黎族自治县','乐东黎族自治县','陵水黎族自治县','保亭黎族苗族自治县','琼中黎族苗族自治县','重庆市','成都市','自贡市','攀枝花市','泸州市',
                    '德阳市','绵阳市','广元市','遂宁市','内江市','乐山市','南充市','眉山市','宜宾市','广安市','达州市','雅安市','巴中市','资阳市','阿坝藏族羌族自治州',
                    '甘孜藏族自治州','凉山彝族自治州','贵阳市','六盘水市','遵义市','安顺市','铜仁市','黔西南布依族苗族自治州','毕节市','黔东南苗族侗族自治州',
                    '黔南布依族苗族自治州','昆明市','曲靖市','玉溪市','保山市','昭通市','丽江市','普洱市','临沧市','楚雄彝族自治州','红河哈尼族彝族自治州',
                    '文山壮族苗族自治州','西双版纳傣族自治州','大理白族自治州','德宏傣族景颇族自治州','怒江傈僳族自治州','迪庆藏族自治州','拉萨市','昌都地区',
                    '山南地区','日喀则地区','那曲地区','阿里地区','林芝地区','西安市','铜川市','宝鸡市','咸阳市','渭南市','延安市','汉中市','榆林市','安康市','商洛市',
                    '兰州市','嘉峪关市','金昌市','白银市','天水市','武威市','张掖市','平凉市','酒泉市','庆阳市','定西市','陇南市','临夏回族自治州','甘南藏族自治州','西宁市',
                    '海东市','海北藏族自治州','黄南藏族自治州','海南藏族自治州','果洛藏族自治州','玉树藏族自治州','海西蒙古族藏族自治州','银川市','石嘴山市','吴忠市',
                    '固原市','中卫市','乌鲁木齐市','克拉玛依市','吐鲁番地区','哈密地区','昌吉回族自治州','博尔塔拉蒙古自治州','巴音郭楞蒙古自治州','阿克苏地区',
                    '克孜勒苏柯尔克孜自治州','喀什地区','和田地区','伊犁哈萨克自治州','塔城地区','阿勒泰地区','石河子市','阿拉尔市','图木舒克市','五家渠市',
                    '台北市','高雄市','台南市','台中市','金门县','南投县','基隆市','新竹市','嘉义市','新北市','宜兰县','新竹县','桃园县','苗栗县','彰化县',
                    '嘉义县','云林县','屏东县','台东县','花莲县','澎湖县','连江县','香港岛','九龙','新界','澳门半岛','离岛',]
            tmp = pd.DataFrame()
            tmp[column] = df[column]
            tmp['sheng_check'] = df[column].apply(lambda x: True if x in shi else False)
            df = pd.merge(df, tmp)
            return df
        except:
            print('check address error')
            df = pd.DataFrame()
            return df

    #暂时不写xian
    def __check_xian(self):
        pass


# -*- coding: utf-8 -*-
import pandas as pd
import sqlalchemy as sql
from pykafka import KafkaClient
import json

import csv
'''
@author:Hao Qi

@version:0.0.1
'''
"""
Copyright © 2020 <copyright holders>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the “Software”), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""


# 数据输入类，读取各形式的数据，返回pandas形式的数据对象，DataFrame


class input(object):
    #输入csv格式数据
    @staticmethod
    def read_csv(filepath: str, encoding: str):
        # pandas读取csv文件返回DataFrame对象
        try:
            file = pd.read_csv(filepath, encoding=encoding)
            return file
        except:
            # 读取错误打印读取错误信息，返回空DataFrame
            print("csv file open error")
            file = pd.DataFrame()
            return file
    #输入mysql数据库格式数据
    def read_database(self, host: str,port: str, user: str, password: str, database: str, table: str):
        try:
            con = self.__get_con(host,user,password,port,database)
            table = pd.read_sql_table(table,con)
            return table
        except:
            print('connect database error')
            df = pd.DataFrame()
            return df
    #输入kafka数据（流式数据，接收json格式数据）输出csv文件，以追加数据的方式输出数据
    def read_kafka(self, kfk_host, zk_host,topic,save_path):
        #host = '192.168.10.217:9092'
        #zk_host='192.168.10.217:2181'
        client = KafkaClient(hosts=kfk_host, zookeeper_hosts=zk_host)
        print(client.topics)
        # 消费者
        #topic='messagetest'
        global flag
        flag=1
        topic = client.topics[topic.encode()]
        consumer = topic.get_simple_consumer(consumer_group=b'test_group', auto_commit_enable=True,
                                             auto_commit_interval_ms=1,
                                             consumer_id=b'test_id')
        for message in consumer:
            if message is not None:
                #json2csv
                #json = message.value.decode('utf-8')
                print(message.offset, str(eval(message.value.decode('utf-8'))).replace("'",'"'))
                self.__writerow(str(eval(message.value.decode('utf-8'))).replace("'",'"'),save_path,flag)
                flag=2
                #print(message.offset, message.value.decode('utf-8'))

    #获取数据库链接方法，外部不可访问

    def __get_con(self,host,user,password,port,database):
        connect_url='mysql+pymysql://'+user+':'+password+'@'+host+':'+port+'/'+database
        #connection = sql.engine.create_engine('mysql+pymysql://crawl:123456@192.168.10.155:3306/')
        connection = sql.engine.create_engine(connect_url)
        con = connection.connect()
        return con
    #json2csv,json文件转csv文件，追加写入
    def __writerow(self,jsons,csv_path,flag):
        #json_file = json.dumps(jsons)
        csv_file = open(csv_path, 'a', newline='', encoding='utf8')
        writer = csv.writer(csv_file)
        # 读取json数据
        dic_data = json.loads(jsons)

        keys = []
        for dic in dic_data:
            keys.append(dic)
            #keys = dic.keys()
        # 写入列名,flag为True写入列名，为false则不写入

        if (flag==1):
            writer.writerow(keys)
            flag+=1

            #break
        data=[]
        for dic in dic_data:
            for key in keys:
                data.append(str(dic_data[key]))
                if key not in dic:
                    data.append('')

            #writer.writerow(dic.values())
        writer.writerow(data)
        csv_file.close()


if __name__ == '__main__':
    input = input()
    #table = input.read_database('192.168.10.155','3306','crawl','123456','mysql','user')
    #print(table.head())
    #import json
    #jsons = { "name":"AAA" , "age":"17" },{ "name":"BBB" , "age":"18" },{ "name":"CCC" , "age":"19" }
    #csv_path='D:\\project\\dataclean\\test\\d.csv'
    #input.read_kafka('192.168.10.217:9092','192.168.10.217:2181','messagetest',csv_path)






from setuptools import setup, find_packages

setup(
    name = 'lxkjdatacleaner',
    version = '0.0.1',
    keywords='dataclean',
    description = 'a library for data clean',
    license = 'MIT License',
    url = 'https://github.com/qihao123/lxdatacleaner',
    author = 'Hao Qi',
    author_email = '1047355811@qq.com',
    packages = find_packages(),
    include_package_data = True,
    platforms = 'any',
    install_requires = [],
)


# -*- coding: utf-8 -*-
import pandas as pd
import queue
'''
@author:Hao Qi

@version:0.0.1
'''
"""
Copyright © 2020 <copyright holders>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the “Software”), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
#时间所限，0.0.1版本暂不实现flow功能
class flow(object):
    def append(self):
        append()
    def __init__(self):
        self.q=queue.Queue()
        self.flow_name=''
        self.index=[]
        self.options=[]
        self.args=[]
        self.data=[]

    def create_flow(self,flow_name: str,):
        self.flow_name=flow_name


    def delete_work(self):
        pass
    def print_flow(self):
        pass
    def updata_work(self):
        pass
    def run_work(self):
        pass
class append(flow):
    def append_input_work(self,options,df,colunm):
        self.index.append(max(self.index)+1)

        pass
    def append_domain_work(self):
        pass
    def append_check_work(self):
        pass
    def append_output_work(self):
        pass




# -*- coding: utf-8 -*-
import pandas as pd
import sqlalchemy as sql

'''
@author:Hao Qi

@version:0.0.1
'''
"""
Copyright © 2020 <copyright holders>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the “Software”), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""


# 数据输出类，输出各形式的数据，写入数据到文件系统或消息队列或数据库中

class output(object):

    def to_csv(self,df,path,encoding):
        try:
            df.to_csv(path,encoding=encoding)
            print("successed save file to %s",path)
        except:
            print("save file error")

    def to_mysql(self,df,tablename,databasename,host,port,user,password):
        try:
            con = self.__get_con(host,user,password,port,databasename)
            df.to_sql(name=tablename,con=con)
            print('save to mysql successed')
        except:
            print('save to mysql has error')


    def to_json(self,df):
        try:
            res = df.to_json()
            return res
        except:
            print('to json error')
            df = pd.DataFrame()
            return df

    def __get_con(self,host,user,password,port,database):
        connect_url='mysql+pymysql://'+user+':'+password+'@'+host+':'+port+'/'+database
        #connection = sql.engine.create_engine('mysql+pymysql://crawl:123456@192.168.10.155:3306/')
        connection = sql.engine.create_engine(connect_url)
        con = connection.connect()
        return con

if __name__ == '__main__':

    df = pd.read_csv('D:\\project\\dataclean\\test\\a.csv',encoding='utf-8')

    #print(df.head())
    output=output()
    #output.to_mysql(df,'test','test2','192.168.10.155','3306','crawl','123456')
    #print(output.to_json(df))